import glob
import os

from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.core.definitions.schedule import ScheduleExecutionContext
from dagster.core.errors import ScheduleExecutionError, user_code_error_boundary
from dagster.core.storage.tags import check_tags
from dagster.utils import merge_dicts

from .utils import ExecutionMetadata, ExecutionParams, UserFacingGraphQLError, capture_dauphin_error


@capture_dauphin_error
def start_schedule(graphene_info, schedule_name):
    repository = graphene_info.context.get_repository()
    instance = graphene_info.context.instance
    schedule = instance.start_schedule(repository, schedule_name)
    return graphene_info.schema.type_named('RunningScheduleResult')(
        schedule=graphene_info.schema.type_named('RunningSchedule')(
            graphene_info, schedule=schedule
        )
    )


@capture_dauphin_error
def stop_schedule(graphene_info, schedule_name):
    repository = graphene_info.context.get_repository()
    instance = graphene_info.context.instance
    schedule = instance.stop_schedule(repository, schedule_name)
    return graphene_info.schema.type_named('RunningScheduleResult')(
        schedule=graphene_info.schema.type_named('RunningSchedule')(
            graphene_info, schedule=schedule
        )
    )


@capture_dauphin_error
def get_scheduler_or_error(graphene_info):
    repository = graphene_info.context.get_repository()
    instance = graphene_info.context.instance
    if not instance.scheduler:
        raise UserFacingGraphQLError(graphene_info.schema.type_named('SchedulerNotDefinedError')())

    runningSchedules = [
        graphene_info.schema.type_named('RunningSchedule')(graphene_info, schedule=s)
        for s in instance.all_schedules(repository)
    ]

    return graphene_info.schema.type_named('Scheduler')(runningSchedules=runningSchedules)


@capture_dauphin_error
def get_schedule_or_error(graphene_info, schedule_name):
    repository = graphene_info.context.get_repository()
    instance = graphene_info.context.instance
    schedule = instance.get_schedule_by_name(repository, schedule_name)

    if not schedule:
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('ScheduleNotFoundError')(schedule_name=schedule_name)
        )

    return graphene_info.schema.type_named('RunningSchedule')(graphene_info, schedule=schedule)


def execution_params_for_schedule(graphene_info, schedule_def, pipeline_def):
    schedule_context = ScheduleExecutionContext(
        graphene_info.context.instance, graphene_info.context.get_repository()
    )

    # Get environment_dict
    with user_code_error_boundary(
        ScheduleExecutionError,
        lambda: 'Error occurred during the execution of environment_dict_fn for schedule '
        '{schedule_name}'.format(schedule_name=schedule_def.name),
    ):
        environment_dict = schedule_def.get_environment_dict(schedule_context)

    # Get tags
    with user_code_error_boundary(
        ScheduleExecutionError,
        lambda: 'Error occurred during the execution of tags_fn for schedule '
        '{schedule_name}'.format(schedule_name=schedule_def.name),
    ):
        schedule_tags = schedule_def.get_tags(schedule_context)

    pipeline_tags = pipeline_def.tags or {}
    check_tags(pipeline_tags, 'pipeline_tags')
    tags = merge_dicts(pipeline_tags, schedule_tags)

    selector = schedule_def.selector
    mode = schedule_def.mode

    return ExecutionParams(
        selector=selector,
        environment_dict=environment_dict,
        mode=mode,
        execution_metadata=ExecutionMetadata(tags=tags, run_id=None),
        step_keys=None,
    )


def get_dagster_schedule_def(graphene_info, schedule_name):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.str_param(schedule_name, 'schedule_name')
    repository = graphene_info.context.get_repository()
    schedule_definition = repository.get_schedule_def(schedule_name)
    return schedule_definition


def get_dagster_schedule(graphene_info, schedule_name):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.str_param(schedule_name, 'schedule_name')

    repository = graphene_info.context.get_repository()
    instance = graphene_info.context.instance
    if not instance.scheduler:
        raise UserFacingGraphQLError(graphene_info.schema.type_named('SchedulerNotDefinedError')())

    schedule = instance.get_schedule_by_name(repository, schedule_name)
    if not schedule:
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('ScheduleNotFoundError')(schedule_name=schedule_name)
        )

    return schedule


def get_schedule_attempt_filenames(graphene_info, schedule_name):
    instance = graphene_info.context.instance
    repository = graphene_info.context.get_repository()
    log_dir = instance.log_path_for_schedule(repository, schedule_name)
    return glob.glob(os.path.join(log_dir, "*.result"))



from __future__ import absolute_import

import abc
import os
import sys
from queue import Empty

import gevent
import six

from dagster import (
    DagsterInstance,
    ExecutionTargetHandle,
    PipelineDefinition,
    PipelineExecutionResult,
    PipelineRun,
    check,
)
from dagster.core.definitions.executable import InMemoryExecutablePipeline
from dagster.core.errors import DagsterSubprocessError
from dagster.core.events import EngineEventData
from dagster.core.execution.api import execute_run_iterator
from dagster.utils import get_multiprocessing_context, start_termination_thread
from dagster.utils.error import serializable_error_info_from_exc_info


class PipelineExecutionManager(six.with_metaclass(abc.ABCMeta)):
    @abc.abstractmethod
    def execute_pipeline(self, handle, pipeline_def, pipeline_run, instance):
        '''Subclasses must implement this method.'''

    @abc.abstractmethod
    def terminate(self, run_id):
        '''Attempt to terminate a run if possible. Return False if unable to, True if it can.'''

    @abc.abstractmethod
    def can_terminate(self, run_id):
        '''Whether or not this execution manager can terminate the given run_id'''

    @abc.abstractmethod
    def get_active_run_count(self):
        '''The number of actively running execution jobs'''

    @abc.abstractmethod
    def is_active(self, run_id):
        '''Whether a given run_id is actively running'''


class SynchronousExecutionManager(PipelineExecutionManager):
    def __init__(self):
        self._active = set()

    def execute_pipeline(self, _, pipeline_def, pipeline_run, instance):
        check.inst_param(pipeline_def, 'pipeline_def', PipelineDefinition)
        check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
        check.inst_param(instance, 'instance', DagsterInstance)

        event_list = []
        self._active.add(pipeline_run.run_id)
        for event in execute_run_iterator(
            InMemoryExecutablePipeline(pipeline_def), pipeline_run, instance
        ):
            event_list.append(event)
        self._active.remove(pipeline_run.run_id)
        return PipelineExecutionResult(pipeline_def, pipeline_run.run_id, event_list, lambda: None)

    def can_terminate(self, run_id):
        return False

    def terminate(self, run_id):
        return False

    def get_active_run_count(self):
        return len(self._active)

    def is_active(self, run_id):
        return run_id in self._active


SUBPROCESS_TICK = 0.5


class SubprocessExecutionManager(PipelineExecutionManager):
    '''
    This execution manager launches a new process for every pipeline invocation.
    It tries to spawn new processes with clean state whenever possible,
    in order to pick up the latest changes, to not inherit in-memory
    state accumulated from the webserver, and to mimic standalone invocations
    of the CLI as much as possible.

    The exception here is unix variants before python 3.4. Before 3.4
    multiprocessing could not configure process start methods, so it
    falls back to system default. On unix variants that means it forks
    the process. This could lead to subtle behavior changes between
    python 2 and python 3.
    '''

    def __init__(self, instance):
        self._multiprocessing_context = get_multiprocessing_context()
        self._instance = instance
        self._living_process_by_run_id = {}
        self._term_events = {}
        self._processes_lock = self._multiprocessing_context.Lock()

        gevent.spawn(self._clock)

    def _generate_synthetic_error_from_crash(self, run):
        message = 'Pipeline execution process for {run_id} unexpectedly exited.'.format(
            run_id=run.run_id
        )
        self._instance.report_engine_event(message, run, cls=self.__class__)
        self._instance.report_run_failed(run)

    def _living_process_snapshot(self):
        with self._processes_lock:
            return {run_id: process for run_id, process in self._living_process_by_run_id.items()}

    def _clock(self):
        '''
        This function polls the instance to synchronize it with the state of processes managed
        by this manager instance. On every tick (every 0.5 seconds currently) it checks for zombie
        processes
        '''
        while True:
            self._check_for_zombies()
            gevent.sleep(SUBPROCESS_TICK)

    def _check_for_zombies(self):
        '''
        Checks the current index of run_id => process and sees if any of them are dead. If they are,
        it queries the instance to see if the runs are in a proper terminal state (success or
        failure). If not, then we can assume that the underlying process died unexpected and clean
        everything. In either case, the dead process is removed from the run_id => process index.
        '''
        runs_to_clear = []

        living_process_snapshot = self._living_process_snapshot()

        for run_id, process in living_process_snapshot.items():
            if not process.is_alive():
                run = self._instance.get_run_by_id(run_id)
                if not run:  # defensive
                    continue

                runs_to_clear.append(run_id)

                # expected terminal state. it's fine for process to be dead
                if run.is_finished:
                    continue

                # the process died in an unexpected manner. inform the system
                self._generate_synthetic_error_from_crash(run)

        with self._processes_lock:
            for run_to_clear_id in runs_to_clear:
                del self._living_process_by_run_id[run_to_clear_id]
                del self._term_events[run_to_clear_id]

    def check(self):
        '''
        Utility method for pytest to manually kick off zombie cleanup calls (in absence of
        gevent)
        '''
        self._check_for_zombies()

    def execute_pipeline(self, handle, pipeline_def, pipeline_run, instance):
        '''Subclasses must implement this method.'''
        check.inst_param(handle, 'handle', ExecutionTargetHandle)
        term_event = self._multiprocessing_context.Event()
        mp_process = self._multiprocessing_context.Process(
            target=self.__class__.in_mp_process,
            kwargs={
                'handle': handle,
                'pipeline_run': pipeline_run,
                'instance_ref': instance.get_ref(),
                'term_event': term_event,
            },
        )

        instance.report_engine_event(
            'About to start process for pipeline "{pipeline_name}" (run_id: {run_id}).'.format(
                pipeline_name=pipeline_run.pipeline_name, run_id=pipeline_run.run_id
            ),
            pipeline_run,
            engine_event_data=EngineEventData(marker_start='dagit_subprocess_init'),
            cls=self.__class__,
        )
        mp_process.start()

        with self._processes_lock:
            self._living_process_by_run_id[pipeline_run.run_id] = mp_process
            self._term_events[pipeline_run.run_id] = term_event

    def join(self):
        with self._processes_lock:
            for run_id, process in self._living_process_by_run_id.items():
                if process.is_alive():
                    process.join()

                run = self._instance.get_run_by_id(run_id)

                if run and not run.is_finished:
                    self._generate_synthetic_error_from_crash(run)

    def _get_process(self, run_id):
        with self._processes_lock:
            return self._living_process_by_run_id.get(run_id)

    def is_process_running(self, run_id):
        check.str_param(run_id, 'run_id')
        process = self._get_process(run_id)
        return process.is_alive() if process else False

    def can_terminate(self, run_id):
        check.str_param(run_id, 'run_id')

        process = self._get_process(run_id)

        if not process:
            return False

        if not process.is_alive():
            return False

        return True

    def terminate(self, run_id):
        check.str_param(run_id, 'run_id')

        process = self._get_process(run_id)

        if not process:
            return False

        if not process.is_alive():
            return False

        self._term_events[run_id].set()
        process.join()
        return True

    def get_active_run_count(self):
        with self._processes_lock:
            return len(self._living_process_by_run_id)

    def is_active(self, run_id):
        with self._processes_lock:
            return run_id in self._living_process_by_run_id

    @classmethod
    def in_mp_process(cls, handle, pipeline_run, instance_ref, term_event):
        """
        Execute pipeline using message queue as a transport
        """
        run_id = pipeline_run.run_id
        pipeline_name = pipeline_run.pipeline_name

        instance = DagsterInstance.from_ref(instance_ref)
        pid = os.getpid()
        instance.report_engine_event(
            'Started process for pipeline (pid: {pid}).'.format(pid=pid),
            pipeline_run,
            EngineEventData.in_process(pid, marker_end='dagit_subprocess_init'),
            cls,
        )

        start_termination_thread(term_event)

        try:
            handle.build_repository_definition()
            pipeline_def = handle.with_pipeline_name(pipeline_name).build_pipeline_definition()
        except Exception:  # pylint: disable=broad-except
            instance.report_engine_event(
                'Failed attempting to load pipeline "{}"'.format(pipeline_name),
                pipeline_run,
                EngineEventData.engine_error(serializable_error_info_from_exc_info(sys.exc_info())),
                cls,
            )
            return

        try:
            event_list = []
            for event in execute_run_iterator(
                InMemoryExecutablePipeline(
                    pipeline_def.build_sub_pipeline(pipeline_run.selector.solid_subset)
                ),
                pipeline_run,
                instance,
            ):
                event_list.append(event)
            return PipelineExecutionResult(pipeline_def, run_id, event_list, lambda: None)

        # Add a DagsterEvent for unexpected exceptions
        # Explicitly ignore KeyboardInterrupts since they are used for termination
        except DagsterSubprocessError as err:
            if not all(
                [
                    err_info.cls_name == 'KeyboardInterrupt'
                    for err_info in err.subprocess_error_infos
                ]
            ):
                instance.report_engine_event(
                    'An exception was thrown during execution that is likely a framework error, '
                    'rather than an error in user code.',
                    pipeline_run,
                    EngineEventData.engine_error(
                        serializable_error_info_from_exc_info(sys.exc_info())
                    ),
                    cls,
                )
        except Exception:  # pylint: disable=broad-except
            instance.report_engine_event(
                'An exception was thrown during execution that is likely a framework error, '
                'rather than an error in user code.',
                pipeline_run,
                EngineEventData.engine_error(serializable_error_info_from_exc_info(sys.exc_info())),
                cls,
            )
        finally:
            instance.report_engine_event(
                'Process for pipeline exited (pid: {pid}).'.format(pid=pid), pipeline_run, cls=cls,
            )


class QueueingSubprocessExecutionManager(PipelineExecutionManager):
    def __init__(self, instance, max_concurrent_runs):
        check.inst_param(instance, 'instance', DagsterInstance)
        self._delegate = SubprocessExecutionManager(instance)
        self._max_concurrent_runs = check.int_param(max_concurrent_runs, 'max_concurrent_runs')
        self._multiprocessing_context = get_multiprocessing_context()
        self._queue = self._multiprocessing_context.JoinableQueue(maxsize=0)
        gevent.spawn(self._clock)

    def _clock(self):
        while True:
            self._check_queue()
            gevent.sleep(1)

    def _check_queue(self):
        run_count = self._delegate.get_active_run_count()
        if run_count < self._max_concurrent_runs:
            try:
                job_args = self._queue.get(block=False)
            except Empty:
                return
            else:
                self._start_pipeline_execution(job_args)

    def _start_pipeline_execution(self, job_args):
        handle = job_args['handle']
        pipeline_run = job_args['pipeline_run']
        pipeline = handle.build_repository_definition().get_pipeline(pipeline_run.pipeline_name)
        instance = DagsterInstance.from_ref(job_args['instance_ref'])
        self._delegate.execute_pipeline(handle, pipeline, pipeline_run, instance)

    def execute_pipeline(self, handle, pipeline_def, pipeline_run, instance):
        check.inst_param(handle, 'handle', ExecutionTargetHandle)
        check.inst_param(pipeline_def, 'pipeline_def', PipelineDefinition)
        job_args = {
            'handle': handle,
            'pipeline_run': pipeline_run,
            'instance_ref': instance.get_ref(),
        }
        self._queue.put(job_args, block=False)
        self._check_queue()

    def check(self):
        '''
        Utility method for pytest to manually kick off queue check calls
        '''
        self._check_queue()
        self._delegate.check()

    def can_terminate(self, run_id):
        # deal with enqueued shit
        return self._delegate.can_terminate(run_id)

    def terminate(self, run_id):
        # deal with enqueued shit
        return self._delegate.terminate(run_id)

    def get_active_run_count(self):
        return self._delegate.get_active_run_count()

    def is_active(self, run_id):
        return self._delegate.is_active(run_id)



import re
from copy import deepcopy

from .test_expectations import sanitize as sanitize_gql
from .utils import sync_execute_get_events


def sanitize(logs):
    res = deepcopy(logs)
    for log in res:
        if 'message' in log:
            log['message'] = re.sub('(pid: [0-9]*)', '(pid: *****)', log['message'])
            log['message'] = re.sub('in [0-9.]*ms', 'in ***.**ms', log['message'])

    return res


def test_materializations(snapshot):
    logs = sanitize_gql(
        sanitize(
            sync_execute_get_events(
                variables={
                    'executionParams': {
                        'selector': {'name': 'materialization_pipeline'},
                        'mode': 'default',
                    }
                }
            )
        )
    )

    # Remove execution durations from ExecutionStepSuccessEvent messages
    for log in logs:
        if log["__typename"] == "ExecutionStepSuccessEvent":
            log["message"] = re.sub(r'in .*', "in", log["message"])

    materializations = [log for log in logs if log['__typename'] == 'StepMaterializationEvent']
    assert len(materializations) == 1
    mat = materializations[0]['materialization']
    assert mat['label'] == 'all_types'

    text_entry = mat['metadataEntries'][0]
    assert text_entry['__typename'] == 'EventTextMetadataEntry'
    assert text_entry['text']

    text_entry = mat['metadataEntries'][1]
    assert text_entry['__typename'] == 'EventUrlMetadataEntry'
    assert text_entry['url']

    text_entry = mat['metadataEntries'][2]
    assert text_entry['__typename'] == 'EventPathMetadataEntry'
    assert text_entry['path']

    text_entry = mat['metadataEntries'][3]
    assert text_entry['__typename'] == 'EventJsonMetadataEntry'
    assert text_entry['jsonString']

    text_entry = mat['metadataEntries'][4]
    assert text_entry['__typename'] == 'EventPythonArtifactMetadataEntry'
    assert text_entry['module']
    assert text_entry['name']

    text_entry = mat['metadataEntries'][5]
    assert text_entry['__typename'] == 'EventPythonArtifactMetadataEntry'
    assert text_entry['module']
    assert text_entry['name']

    text_entry = mat['metadataEntries'][6]
    assert text_entry['__typename'] == 'EventFloatMetadataEntry'
    assert text_entry['value']

    snapshot.assert_match(logs)



from __future__ import absolute_import

from dagster_graphql import dauphin
from dagster_graphql.schema.runtime_types import to_dauphin_dagster_type

from dagster import check
from dagster.core.snap import (
    ExecutionPlanIndex,
    ExecutionStepInputSnap,
    ExecutionStepOutputSnap,
    ExecutionStepSnap,
    PipelineSnapshot,
)


class DauphinExecutionPlan(dauphin.ObjectType):
    class Meta(object):
        name = 'ExecutionPlan'

    steps = dauphin.non_null_list('ExecutionStep')
    artifactsPersisted = dauphin.NonNull(dauphin.Boolean)

    def __init__(self, execution_plan_index):
        super(DauphinExecutionPlan, self).__init__()
        self._execution_plan_index = check.inst_param(
            execution_plan_index, execution_plan_index, ExecutionPlanIndex
        )

    def resolve_steps(self, _graphene_info):
        return [
            DauphinExecutionStep(
                self._execution_plan_index, self._execution_plan_index.get_step_by_key(step.key),
            )
            for step in self._execution_plan_index.execution_plan_snapshot.steps
        ]

    def resolve_artifactsPersisted(self, _graphene_info):
        return self._execution_plan_index.execution_plan_snapshot.artifacts_persisted


class DauphinExecutionStepOutput(dauphin.ObjectType):
    class Meta(object):
        name = 'ExecutionStepOutput'

    name = dauphin.NonNull(dauphin.String)
    type = dauphin.Field(dauphin.NonNull('RuntimeType'))

    def __init__(self, pipeline_snapshot, step_output_snap):
        super(DauphinExecutionStepOutput, self).__init__()
        self._step_output_snap = check.inst_param(
            step_output_snap, 'step_output_snap', ExecutionStepOutputSnap
        )
        self._pipeline_snapshot = check.inst_param(
            pipeline_snapshot, 'pipeline_snapshot', PipelineSnapshot
        )

    def resolve_name(self, _graphene_info):
        return self._step_output_snap.name

    def resolve_type(self, _graphene_info):
        return to_dauphin_dagster_type(
            self._pipeline_snapshot, self._step_output_snap.dagster_type_key
        )


class DauphinExecutionStepInput(dauphin.ObjectType):
    class Meta(object):
        name = 'ExecutionStepInput'

    name = dauphin.NonNull(dauphin.String)
    type = dauphin.Field(dauphin.NonNull('RuntimeType'))
    dependsOn = dauphin.non_null_list('ExecutionStep')

    def __init__(self, pipeline_snapshot, step_input_snap, execution_plan_index):
        super(DauphinExecutionStepInput, self).__init__()
        self._step_input_snap = check.inst_param(
            step_input_snap, 'step_input_snap', ExecutionStepInputSnap
        )
        self._pipeline_snapshot = check.inst_param(
            pipeline_snapshot, 'pipeline_snapshot', PipelineSnapshot
        )
        self._execution_plan_index = check.inst_param(
            execution_plan_index, 'execution_plan_index', ExecutionPlanIndex
        )

    def resolve_name(self, _graphene_info):
        return self._step_input_snap.name

    def resolve_type(self, _graphene_info):
        return to_dauphin_dagster_type(
            self._pipeline_snapshot, self._step_input_snap.dagster_type_key
        )

    def resolve_dependsOn(self, graphene_info):
        return [
            graphene_info.schema.type_named('ExecutionStep')(
                self._execution_plan_index, self._execution_plan_index.get_step_by_key(key),
            )
            for key in self._step_input_snap.upstream_step_keys
        ]


class DauphinStepKind(dauphin.Enum):
    class Meta(object):
        name = 'StepKind'

    COMPUTE = 'COMPUTE'

    @property
    def description(self):
        # self ends up being the internal class "EnumMeta" in dauphin
        # so we can't do a dictionary lookup which is awesome
        if self == DauphinStepKind.COMPUTE:
            return 'This is the user-defined computation step'
        else:
            return None


class DauphinExecutionStep(dauphin.ObjectType):
    class Meta(object):
        name = 'ExecutionStep'

    key = dauphin.NonNull(dauphin.String)
    inputs = dauphin.non_null_list('ExecutionStepInput')
    outputs = dauphin.non_null_list('ExecutionStepOutput')
    solidHandleID = dauphin.NonNull(dauphin.String)
    kind = dauphin.NonNull('StepKind')
    metadata = dauphin.non_null_list('MetadataItemDefinition')

    def __init__(self, execution_plan_index, execution_step_snap):
        super(DauphinExecutionStep, self).__init__()
        self._execution_plan_index = check.inst_param(
            execution_plan_index, 'execution_plan_index', ExecutionPlanIndex
        )
        self._plan_snapshot = execution_plan_index.execution_plan_snapshot
        self._step_snap = check.inst_param(
            execution_step_snap, 'execution_step_snap', ExecutionStepSnap
        )

    def resolve_metadata(self, graphene_info):
        return [
            graphene_info.schema.type_named('MetadataItemDefinition')(key=mdi.key, value=mdi.value)
            for mdi in self._step_snap.metadata_items
        ]

    def resolve_inputs(self, graphene_info):
        return [
            graphene_info.schema.type_named('ExecutionStepInput')(
                self._execution_plan_index.pipeline_index.pipeline_snapshot,
                inp,
                self._execution_plan_index,
            )
            for inp in self._step_snap.inputs
        ]

    def resolve_outputs(self, graphene_info):
        return [
            graphene_info.schema.type_named('ExecutionStepOutput')(
                self._execution_plan_index.pipeline_index.pipeline_snapshot, out,
            )
            for out in self._step_snap.outputs
        ]

    def resolve_key(self, _graphene_info):
        return self._step_snap.key

    def resolve_solidHandleID(self, _graphene_info):
        return self._step_snap.solid_handle_id

    def resolve_kind(self, _graphene_info):
        return self._step_snap.kind



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['test_full_pipeline_reexecution_fs_storage 1'] = {
    'startPipelineExecution': {
        '__typename': 'StartPipelineRunSuccess',
        'run': {
            'pipeline': {
                'name': 'csv_hello_world'
            },
            'tags': [
            ]
        }
    }
}

snapshots['test_full_pipeline_reexecution_in_memory_storage 1'] = {
    'startPipelineExecution': {
        '__typename': 'StartPipelineRunSuccess',
        'run': {
            'pipeline': {
                'name': 'csv_hello_world'
            },
            'tags': [
            ]
        }
    }
}



import yaml
from dagster_graphql import dauphin
from dagster_graphql.implementation.fetch_runs import get_runs
from dagster_graphql.schema.errors import (
    DauphinPartitionSetNotFoundError,
    DauphinPipelineNotFoundError,
    DauphinPythonError,
)

from dagster import check
from dagster.core.definitions.partition import Partition, PartitionSetDefinition
from dagster.core.storage.pipeline_run import PipelineRunsFilter


class DauphinPartition(dauphin.ObjectType):
    class Meta(object):
        name = 'Partition'

    name = dauphin.NonNull(dauphin.String)
    partition_set_name = dauphin.NonNull(dauphin.String)
    solid_subset = dauphin.List(dauphin.NonNull(dauphin.String))
    mode = dauphin.NonNull(dauphin.String)
    environmentConfigYaml = dauphin.NonNull(dauphin.String)
    tags = dauphin.non_null_list('PipelineTag')
    runs = dauphin.non_null_list('PipelineRun')

    def __init__(self, partition, partition_set):
        self._partition = check.inst_param(partition, 'partition', Partition)
        self._partition_set = check.inst_param(
            partition_set, 'partition_set', PartitionSetDefinition
        )

        super(DauphinPartition, self).__init__(
            name=partition.name,
            partition_set_name=partition_set.name,
            solid_subset=partition_set.solid_subset,
            mode=partition_set.mode,
        )

    def resolve_environmentConfigYaml(self, _):
        environment_dict = self._partition_set.environment_dict_for_partition(self._partition)
        return yaml.dump(environment_dict, default_flow_style=False)

    def resolve_tags(self, graphene_info):
        return [
            graphene_info.schema.type_named('PipelineTag')(key=key, value=value)
            for key, value in self._partition_set.tags_for_partition(self._partition).items()
        ]

    def resolve_runs(self, graphene_info):
        runs_filter = PipelineRunsFilter(
            tags={
                'dagster/partition_set': self._partition_set.name,
                'dagster/partition': self._partition.name,
            }
        )
        return get_runs(graphene_info, runs_filter)


class DauphinPartitions(dauphin.ObjectType):
    class Meta(object):
        name = 'Partitions'

    results = dauphin.non_null_list('Partition')


class DauphinPartitionSet(dauphin.ObjectType):
    class Meta(object):
        name = 'PartitionSet'

    name = dauphin.NonNull(dauphin.String)
    pipeline_name = dauphin.NonNull(dauphin.String)
    solid_subset = dauphin.List(dauphin.NonNull(dauphin.String))
    mode = dauphin.NonNull(dauphin.String)
    partitions = dauphin.Field(
        dauphin.NonNull('Partitions'), cursor=dauphin.String(), limit=dauphin.Int(),
    )

    def __init__(self, partition_set):
        self._partition_set = check.inst_param(
            partition_set, 'partition_set', PartitionSetDefinition
        )

        super(DauphinPartitionSet, self).__init__(
            name=partition_set.name,
            pipeline_name=partition_set.pipeline_name,
            solid_subset=partition_set.solid_subset,
            mode=partition_set.mode,
        )

    def resolve_partitions(self, graphene_info, **kwargs):
        partitions = self._partition_set.get_partitions()

        cursor = kwargs.get("cursor")
        limit = kwargs.get("limit")
        if cursor and limit:
            index = next(
                (idx for (idx, partition) in enumerate(partitions) if partition.name == cursor),
                None,
            )
            partitions = partitions[index : min(len(partitions), index + limit)]
        elif limit:
            partitions = partitions[: min(len(partitions), limit)]

        return graphene_info.schema.type_named('Partitions')(
            results=[
                graphene_info.schema.type_named('Partition')(
                    partition=partition, partition_set=self._partition_set
                )
                for partition in partitions
            ]
        )


class DapuphinPartitionSetOrError(dauphin.Union):
    class Meta(object):
        name = 'PartitionSetOrError'
        types = ('PartitionSet', DauphinPartitionSetNotFoundError, DauphinPythonError)


class DauphinPartitionSets(dauphin.ObjectType):
    class Meta(object):
        name = 'PartitionSets'

    results = dauphin.non_null_list('PartitionSet')


class DauphinPartitionSetsOrError(dauphin.Union):
    class Meta(object):
        name = 'PartitionSetsOrError'
        types = (DauphinPartitionSets, DauphinPipelineNotFoundError, DauphinPythonError)



import abc

import six

from dagster import ExecutionTargetHandle, check
from dagster.core.definitions.partition import PartitionScheduleDefinition
from dagster.core.execution.api import create_execution_plan
from dagster.core.instance import DagsterInstance
from dagster.core.snap import (
    ActiveRepositoryData,
    ExecutionPlanIndex,
    RepositoryIndex,
    active_repository_data_from_def,
    snapshot_from_execution_plan,
)

from .external import ExternalPipeline
from .pipeline_execution_manager import PipelineExecutionManager
from .reloader import Reloader


class DagsterGraphQLContext(six.with_metaclass(abc.ABCMeta)):
    def __init__(self, active_repository_data, instance):
        self.active_repository_data = check.inst_param(
            active_repository_data, 'active_repository_data', ActiveRepositoryData
        )
        self._repository_index = RepositoryIndex(active_repository_data)
        self._instance = check.inst_param(instance, 'instance', DagsterInstance)

    def get_repository_index(self):
        return self._repository_index

    @property
    def instance(self):
        return self._instance

    @abc.abstractproperty
    def is_reload_supported(self):
        pass

    def has_external_pipeline(self, name):
        check.str_param(name, 'name')
        return self._repository_index.has_pipeline(name)

    def get_external_pipeline(self, name):
        check.str_param(name, 'name')
        return ExternalPipeline(
            self._repository_index.get_pipeline_index(name),
            self.active_repository_data.get_active_pipeline_data(name),
        )

    def get_all_external_pipelines(self):
        return list(
            map(
                lambda pi: ExternalPipeline(
                    pi, self.active_repository_data.get_active_pipeline_data(pi.name)
                ),
                self._repository_index.get_pipeline_indices(),
            )
        )

    @abc.abstractmethod
    def get_external_pipeline_subset(self, name, solid_subset):
        pass

    @abc.abstractmethod
    def create_execution_plan_index(
        self, external_pipeline, environment_dict, mode, step_keys_to_execute
    ):
        pass


class DagsterGraphQLOutOfProcessRepositoryContext(DagsterGraphQLContext):
    def __init__(self, active_repository_data, execution_manager, instance, version=None):
        super(DagsterGraphQLOutOfProcessRepositoryContext, self).__init__(
            active_repository_data=active_repository_data, instance=instance
        )
        self.execution_manager = check.inst_param(
            execution_manager, 'pipeline_execution_manager', PipelineExecutionManager
        )
        self.version = version

    @property
    def is_reload_supported(self):
        return False

    def get_external_pipeline_subset(self, name, solid_subset):
        raise NotImplementedError('Not yet supported out of process')

    def create_execution_plan_index(
        self, external_pipeline, environment_dict, mode, step_keys_to_execute
    ):
        raise NotImplementedError('Not yet supported out of process')


class DagsterGraphQLInProcessRepositoryContext(DagsterGraphQLContext):
    def __init__(self, handle, execution_manager, instance, reloader=None, version=None):
        self.repository_definition = handle.build_repository_definition()
        super(DagsterGraphQLInProcessRepositoryContext, self).__init__(
            active_repository_data=active_repository_data_from_def(self.repository_definition),
            instance=instance,
        )
        self._handle = check.inst_param(handle, 'handle', ExecutionTargetHandle)
        self.reloader = check.opt_inst_param(reloader, 'reloader', Reloader)
        self.execution_manager = check.inst_param(
            execution_manager, 'pipeline_execution_manager', PipelineExecutionManager
        )
        self.version = version
        self._cached_pipelines = {}

    def get_handle(self):
        return self._handle

    def get_partition_set(self, partition_set_name):
        return next(
            (
                partition_set
                for partition_set in self.get_all_partition_sets()
                if partition_set.name == partition_set_name
            ),
            None,
        )

    def get_all_partition_sets(self):
        return self.repository_definition.partition_set_defs + [
            schedule_def.get_partition_set()
            for schedule_def in self.repository_definition.schedule_defs
            if isinstance(schedule_def, PartitionScheduleDefinition)
        ]

    def get_repository(self):
        return self.repository_definition

    def get_pipeline(self, pipeline_name):
        if not pipeline_name in self._cached_pipelines:
            self._cached_pipelines[pipeline_name] = self._build_pipeline(pipeline_name)

        return self._cached_pipelines[pipeline_name]

    def _build_pipeline(self, pipeline_name):
        orig_handle = self.get_handle()
        if orig_handle.is_resolved_to_pipeline:
            pipeline_def = orig_handle.build_pipeline_definition()
            check.invariant(
                pipeline_def.name == pipeline_name,
                '''Dagster GraphQL Context resolved pipeline with name {handle_pipeline_name},
                couldn't resolve {pipeline_name}'''.format(
                    handle_pipeline_name=pipeline_def.name, pipeline_name=pipeline_name
                ),
            )
            return pipeline_def
        return self.get_handle().with_pipeline_name(pipeline_name).build_pipeline_definition()

    @property
    def is_reload_supported(self):
        return self.reloader and self.reloader.is_reload_supported

    def get_external_pipeline_subset(self, name, solid_subset):
        check.str_param(name, 'name')
        check.list_param(solid_subset, 'solid_subset', of_type=str)

        return ExternalPipeline.from_pipeline_def(
            self.get_pipeline(name).build_sub_pipeline(solid_subset)
        )

    def create_execution_plan_index(
        self, external_pipeline, environment_dict, mode, step_keys_to_execute=None
    ):
        check.inst_param(external_pipeline, 'external_pipeline', ExternalPipeline)
        check.dict_param(environment_dict, 'environment_dict')
        check.str_param(mode, 'mode')
        check.opt_list_param(step_keys_to_execute, 'step_keys_to_execute', of_type=str)

        return ExecutionPlanIndex(
            execution_plan_snapshot=snapshot_from_execution_plan(
                create_execution_plan(
                    pipeline=self.get_pipeline(external_pipeline.name),
                    environment_dict=environment_dict,
                    mode=mode,
                    step_keys_to_execute=step_keys_to_execute,
                ),
                external_pipeline.pipeline_snapshot_id,
            ),
            pipeline_index=external_pipeline.pipeline_index,
        )



import time

from dagster_graphql.test.utils import define_context_for_file, execute_dagster_graphql

from dagster import (
    Materialization,
    Output,
    RepositoryDefinition,
    execute_pipeline,
    pipeline,
    seven,
    solid,
)
from dagster.core.instance import DagsterInstance, InstanceType
from dagster.core.storage.event_log import InMemoryEventLogStorage
from dagster.core.storage.local_compute_log_manager import NoOpComputeLogManager
from dagster.core.storage.root import LocalArtifactStorage
from dagster.core.storage.runs import InMemoryRunStorage

GET_ASSET_KEY_QUERY = '''
{
    assetsOrError {
        __typename
        ...on AssetConnection {
            nodes {
                key
            }
        }
    }
}
'''

GET_ASSET_MATERIALIZATION = '''
    query AssetQuery($assetKey: String!) {
        assetOrError(assetKey: $assetKey) {
            ... on Asset {
                assetMaterializations(limit: 1) {
                    materializationEvent {
                        materialization {
                            label
                        }
                    }
                }
            }
        }
    }
'''

GET_ASSET_RUNS = '''
    query AssetRunsQuery($assetKey: String!) {
        assetOrError(assetKey: $assetKey) {
            ... on Asset {
                runs {
                    runId
                }
            }
        }
    }
'''


def get_instance(temp_dir):
    return DagsterInstance(
        instance_type=InstanceType.EPHEMERAL,
        local_artifact_storage=LocalArtifactStorage(temp_dir),
        run_storage=InMemoryRunStorage(),
        event_storage=InMemoryEventLogStorage(),
        compute_log_manager=NoOpComputeLogManager(temp_dir),
    )


def asset_repo():
    @solid
    def solid_a(_):
        yield Materialization(asset_key='a', label='a')
        yield Output(1)

    @solid
    def solid_b(_, num):
        yield Materialization(asset_key='b', label='b')
        time.sleep(0.1)
        yield Materialization(asset_key='c', label='c')
        yield Output(num)

    @pipeline
    def single_asset_pipeline():
        solid_a()

    @pipeline
    def multi_asset_pipeline():
        solid_b(solid_a())

    return RepositoryDefinition(
        'asset_repo', pipeline_defs=[single_asset_pipeline, multi_asset_pipeline]
    )


def test_get_all_asset_keys(snapshot):
    with seven.TemporaryDirectory() as temp_dir:
        instance = get_instance(temp_dir)
        repo = asset_repo()
        execute_pipeline(repo.get_pipeline('multi_asset_pipeline'), instance=instance)
        context = define_context_for_file(__file__, 'asset_repo', instance)
        result = execute_dagster_graphql(context, GET_ASSET_KEY_QUERY)
        assert result.data
        snapshot.assert_match(result.data)


def test_get_asset_key_materialization(snapshot):
    with seven.TemporaryDirectory() as temp_dir:
        instance = get_instance(temp_dir)
        repo = asset_repo()
        execute_pipeline(repo.get_pipeline('single_asset_pipeline'), instance=instance)
        context = define_context_for_file(__file__, 'asset_repo', instance)
        result = execute_dagster_graphql(
            context, GET_ASSET_MATERIALIZATION, variables={'assetKey': 'a'}
        )
        assert result.data
        snapshot.assert_match(result.data)


def test_get_asset_runs():
    with seven.TemporaryDirectory() as temp_dir:
        instance = get_instance(temp_dir)
        repo = asset_repo()
        single_run_id = execute_pipeline(
            repo.get_pipeline('single_asset_pipeline'), instance=instance
        ).run_id
        multi_run_id = execute_pipeline(
            repo.get_pipeline('multi_asset_pipeline'), instance=instance
        ).run_id
        context = define_context_for_file(__file__, 'asset_repo', instance)
        result = execute_dagster_graphql(context, GET_ASSET_RUNS, variables={'assetKey': 'a'})
        assert result.data
        fetched_runs = [run['runId'] for run in result.data['assetOrError']['runs']]
        assert len(fetched_runs) == 2
        assert multi_run_id in fetched_runs
        assert single_run_id in fetched_runs



from dagster_graphql.implementation.context import DagsterGraphQLInProcessRepositoryContext
from dagster_graphql.implementation.external import (
    get_external_pipeline_or_raise,
    get_external_pipeline_subset_or_raise,
)
from dagster_graphql.schema.pipelines import DauphinPipeline, DauphinPipelineSnapshot
from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.core.definitions.pipeline import ExecutionSelector
from dagster.core.snap import PipelineIndex

from .utils import UserFacingGraphQLError, capture_dauphin_error


def get_pipeline_snapshot(graphene_info, snapshot_id):
    check.str_param(snapshot_id, 'snapshot_id')
    pipeline_snapshot = graphene_info.context.instance.get_pipeline_snapshot(snapshot_id)
    return DauphinPipelineSnapshot(PipelineIndex(pipeline_snapshot))


@capture_dauphin_error
def get_pipeline_snapshot_or_error_from_pipeline_name(graphene_info, pipeline_name):
    check.str_param(pipeline_name, 'pipeline_name')
    return DauphinPipelineSnapshot(
        get_external_pipeline_or_raise(graphene_info, pipeline_name).pipeline_index
    )


@capture_dauphin_error
def get_pipeline_snapshot_or_error_from_snapshot_id(graphene_info, snapshot_id):
    check.str_param(snapshot_id, 'snapshot_id')
    return _get_dauphin_pipeline_snapshot_from_instance(graphene_info.context.instance, snapshot_id)


# extracted this out to test
def _get_dauphin_pipeline_snapshot_from_instance(instance, snapshot_id):
    from dagster_graphql.schema.errors import DauphinPipelineSnapshotNotFoundError

    if not instance.has_pipeline_snapshot(snapshot_id):
        raise UserFacingGraphQLError(DauphinPipelineSnapshotNotFoundError(snapshot_id))

    pipeline_snapshot = instance.get_pipeline_snapshot(snapshot_id)

    if not pipeline_snapshot:
        # Either a temporary error or it has been deleted in the interim
        raise UserFacingGraphQLError(DauphinPipelineSnapshotNotFoundError(snapshot_id))

    return DauphinPipelineSnapshot(PipelineIndex(pipeline_snapshot))


@capture_dauphin_error
def get_pipeline_or_error(graphene_info, selector):
    '''Returns a DauphinPipelineOrError.'''
    return get_dauphin_pipeline_from_selector(graphene_info, selector)


def get_pipeline_or_raise(graphene_info, selector):
    '''Returns a DauphinPipeline or raises a UserFacingGraphQLError if one cannot be retrieved
    from the selector, e.g., the pipeline is not present in the loaded repository.'''
    return get_dauphin_pipeline_from_selector(graphene_info, selector)


def get_pipeline_reference_or_raise(graphene_info, selector):
    '''Returns a DauphinPipelineReference or raises a UserFacingGraphQLError if a pipeline
    reference cannot be retrieved from the selector, e.g, a UserFacingGraphQLError that wraps an
    InvalidSubsetError.'''
    return get_dauphin_pipeline_reference_from_selector(graphene_info, selector)


def get_dauphin_pipeline_reference_from_selector(graphene_info, selector):
    from ..schema.errors import DauphinPipelineNotFoundError, DauphinInvalidSubsetError

    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)
    try:
        return get_dauphin_pipeline_from_selector(graphene_info, selector)
    except UserFacingGraphQLError as exc:
        if (
            isinstance(exc.dauphin_error, DauphinPipelineNotFoundError)
            or
            # At this time DauphinPipeline represents a potentially subsetted
            # pipeline so if the solids used to subset no longer exist
            # we can't return the correct instance so we fallback to
            # UnknownPipeline
            isinstance(exc.dauphin_error, DauphinInvalidSubsetError)
        ):
            return graphene_info.schema.type_named('UnknownPipeline')(selector.name)

        raise


@capture_dauphin_error
def get_pipelines_or_error(graphene_info):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    return get_pipelines_or_raise(graphene_info)


def get_pipelines_or_raise(graphene_info):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    dauphin_pipelines = list(
        map(DauphinPipeline, graphene_info.context.get_all_external_pipelines())
    )
    return graphene_info.schema.type_named('PipelineConnection')(
        nodes=sorted(dauphin_pipelines, key=lambda pipeline: pipeline.name)
    )


def get_dauphin_pipeline_from_selector(graphene_info, selector):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)

    if not isinstance(graphene_info.context, DagsterGraphQLInProcessRepositoryContext):
        # TODO: Support solid sub selection.
        check.invariant(
            not selector.solid_subset,
            desc="DagsterGraphQLOutOfProcessRepositoryContext doesn't support pipeline sub-selection.",
        )

    return DauphinPipeline(
        get_external_pipeline_subset_or_raise(graphene_info, selector.name, selector.solid_subset)
    )


def get_pipeline_def_from_selector(graphene_info, selector):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', ExecutionSelector)

    check.invariant(
        isinstance(graphene_info.context, DagsterGraphQLInProcessRepositoryContext),
        'Can only get definition objects in process',
    )

    pipeline_name = selector.name

    # for error check of pipeline existence
    get_external_pipeline_or_raise(graphene_info, pipeline_name)

    orig_pipeline_def = graphene_info.context.get_pipeline(pipeline_name)

    if not selector.solid_subset:
        return orig_pipeline_def

    # for error checking
    get_external_pipeline_subset_or_raise(graphene_info, selector.name, selector.solid_subset)
    return orig_pipeline_def.build_sub_pipeline(selector.solid_subset)



from __future__ import absolute_import

import sys

from dagster_graphql.schema.errors import DauphinPipelineConfigValidationInvalid
from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.config.validate import validate_config
from dagster.core.definitions.environment_schema import create_environment_schema
from dagster.core.errors import DagsterInvalidConfigError, DagsterLaunchFailedError
from dagster.core.events import EngineEventData
from dagster.core.execution.api import create_execution_plan
from dagster.core.snap.execution_plan_snapshot import snapshot_from_execution_plan
from dagster.utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info

from ..fetch_pipelines import get_pipeline_def_from_selector
from ..fetch_runs import get_validated_config
from ..utils import ExecutionMetadata, ExecutionParams, capture_dauphin_error
from .utils import (
    _check_start_pipeline_execution_errors,
    get_step_keys_to_execute,
    pipeline_run_args_from_execution_params,
)


@capture_dauphin_error
def launch_pipeline_reexecution(graphene_info, execution_params):
    return _launch_pipeline_execution(graphene_info, execution_params, is_reexecuted=True)


@capture_dauphin_error
def launch_pipeline_execution(graphene_info, execution_params):
    return _launch_pipeline_execution(graphene_info, execution_params)


def _launch_pipeline_execution(graphene_info, execution_params, is_reexecuted=False):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(execution_params, 'execution_params', ExecutionParams)

    if is_reexecuted:
        # required fields for re-execution
        execution_metadata = check.inst_param(
            execution_params.execution_metadata, 'execution_metadata', ExecutionMetadata
        )
        check.str_param(execution_metadata.root_run_id, 'root_run_id')
        check.str_param(execution_metadata.parent_run_id, 'parent_run_id')

    instance = graphene_info.context.instance
    run_launcher = instance.run_launcher

    if run_launcher is None:
        return graphene_info.schema.type_named('RunLauncherNotDefinedError')()

    pipeline_def = get_pipeline_def_from_selector(graphene_info, execution_params.selector)

    get_validated_config(
        pipeline_def,
        environment_dict=execution_params.environment_dict,
        mode=execution_params.mode,
    )

    execution_plan = create_execution_plan(
        pipeline_def, execution_params.environment_dict, mode=execution_params.mode,
    )

    _check_start_pipeline_execution_errors(graphene_info, execution_params, execution_plan)

    pipeline_run = instance.create_run(
        pipeline_snapshot=pipeline_def.get_pipeline_snapshot(),
        execution_plan_snapshot=snapshot_from_execution_plan(
            execution_plan, pipeline_def.get_pipeline_snapshot_id()
        ),
        **pipeline_run_args_from_execution_params(
            execution_params, get_step_keys_to_execute(instance, pipeline_def, execution_params),
        )
    )

    run = instance.launch_run(pipeline_run.run_id)

    return graphene_info.schema.type_named('LaunchPipelineRunSuccess')(
        run=graphene_info.schema.type_named('PipelineRun')(run)
    )


def _launch_pipeline_execution_for_created_run(graphene_info, run_id):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.str_param(run_id, 'run_id')

    # First retrieve the pipeline run
    instance = graphene_info.context.instance
    pipeline_run = instance.get_run_by_id(run_id)
    if not pipeline_run:
        return graphene_info.schema.type_named('PipelineRunNotFoundError')(run_id)

    pipeline_def = get_pipeline_def_from_selector(graphene_info, pipeline_run.selector)

    # Run config valudation
    # If there are any config errors, then inject them into the event log
    environment_schema = create_environment_schema(pipeline_def, pipeline_run.mode)
    validated_config = validate_config(
        environment_schema.environment_type, pipeline_run.environment_dict
    )
    if not validated_config.success:
        # If the config is invalid, we construct a DagsterInvalidConfigError exception and
        # insert it into the event log. We also return a PipelineConfigValidationInvalid user facing
        # graphql error.

        # We currently re-use the engine events machinery to add the error to the event log, but
        # may need to create a new event type and instance method to handle these erros.
        invalid_config_exception = DagsterInvalidConfigError(
            'Error in config for pipeline {}'.format(pipeline_def.name),
            validated_config.errors,
            pipeline_run.environment_dict,
        )

        instance.report_engine_event(
            str(invalid_config_exception.message),
            pipeline_run,
            EngineEventData.engine_error(
                SerializableErrorInfo(
                    invalid_config_exception.message,
                    [],
                    DagsterInvalidConfigError.__class__.__name__,
                    None,
                )
            ),
        )

        instance.report_run_failed(pipeline_run)

        return DauphinPipelineConfigValidationInvalid.for_validation_errors(
            pipeline_def.get_pipeline_index(), validated_config.errors
        )

    try:
        pipeline_run = instance.launch_run(pipeline_run.run_id)
    except DagsterLaunchFailedError:
        error = serializable_error_info_from_exc_info(sys.exc_info())
        instance.report_engine_event(
            error.message, pipeline_run, EngineEventData.engine_error(error),
        )
        instance.report_run_failed(pipeline_run)

    return graphene_info.schema.type_named('LaunchPipelineRunSuccess')(
        run=graphene_info.schema.type_named('PipelineRun')(pipeline_run)
    )
